#define DEF_DISPLAY                 ""
#define DEF_FG                      "cyan4"
#define DEF_BG                      "grey50"
#define DEF_FC                      "Blue"
#define DEF_TERM                    "xterm"
#define DEF_BW                      0
#define DEF_DL                      1
#define MOUSERAISE                  1

#define DEF_KEY_NEXT_CLIENT         XK_Tab
#define DEF_KEY_NEW_TERM            XK_Return
#define DEF_KEY_KILL_CLIENT         XK_Escape
#define DEF_KEY_RAISE_CLIENT        XK_f
#define DEF_KEY_LOWER_CLIENT        XK_b
#define DEF_KEY_START_PASSTHROUGH   XK_v
#define DEF_KEY_MOVE_WIN_UP         XK_k
#define DEF_KEY_MOVE_WIN_DOWN       XK_j
#define DEF_KEY_MOVE_WIN_LEFT       XK_h
#define DEF_KEY_MOVE_WIN_RIGHT      XK_l
#define DEF_KEY_EXPAND_WIN_Y        XK_i
#define DEF_KEY_CONTRACT_WIN_Y      XK_o
#define DEF_KEY_EXPAND_WIN_X        XK_p
#define DEF_KEY_CONTRACT_WIN_X      XK_u
#define DEF_KEY_HORIZONTAL_TOGGLE   XK_x
#define DEF_KEY_VERTICAL_TOGGLE     XK_y
#define DEF_KEY_MAXIMIZE_TOGGLE     XK_m
#define DEF_KEY_LOCK_WINDOW         XK_equal
#define DEF_KEY_GOTO_VDESK_0        XK_1
#define DEF_KEY_GOTO_VDESK_1        XK_2
#define DEF_KEY_GOTO_VDESK_2        XK_3
#define DEF_KEY_GOTO_VDESK_3        XK_4
#define DEF_KEY_GOTO_VDESK_4        XK_5
#define DEF_KEY_GOTO_VDESK_5        XK_6
#define DEF_KEY_GOTO_VDESK_6        XK_7
#define DEF_KEY_GOTO_VDESK_7        XK_8
#define DEF_KEY_NEXT_VDESK          XK_Right
#define DEF_KEY_PREV_VDESK          XK_Left
#define DEF_KEY_TOP_LEFT            XK_bracketleft
#define DEF_KEY_TOP_RIGHT           XK_bracketright
#define DEF_KEY_BOTTOM_LEFT         XK_semicolon
#define DEF_KEY_BOTTOM_RIGHT        XK_apostrophe
#define DEF_KEY_TOGGLE_FOCUS        XK_z
#define DEF_KEY_QUIT_WM             XK_End
